#!/bin/sh 

unset QTDIR

QTDIR="/usr/lib/qt4"
SOURCE="../src"
PLATFORM="linux-32-gcc-release"

while [ "$*" != "" ]; do
  a=$1
  shift
  
  case $a in 
  -qt) 
    QTDIR=$1
    shift
    ;;
  -platform) 
    PLATFORM=$1
    shift
    ;;
  -help)
    echo "usage: build [options]"
    echo ""
    echo "options:"
    echo "  -qt <dir>        Set Qt installation directory (default '/usr/lib/qt')"
    echo "  -platform <key>  Select a platform"
    echo ""
    echo "Available platforms are:"
    for pf in `ls -1 config/*.conf.*`; do
      echo $pf | sed "s/config\/Makefile\.conf\./  /"
    done
    exit 0
    ;;
  *)
    echo "*** ERROR: unknown option $a (use -help to show options)"
    exit 1
    ;;
  esac

done

echo "Qt installation directory is $QTDIR"
echo "Platform is $PLATFORM"

EXEC_NAME=klayout

gcc_path=`type gcc 2>/dev/null`
if [ "x$gcc_path" = "x" ]; then
  echo "*** ERROR: unable to locate gcc in path"
  exit 1
fi

if [ ! -r config/Makefile.conf.$PLATFORM ]; then
  echo "*** ERROR: unknown platform $PLATFORM"
  exit 1
fi

if [ ! -r $QTDIR/include/QtCore/Qt ]; then
  echo "*** ERROR: no Qt headers in $QTDIR/include"
  exit 1
fi

if [ ! -r $QTDIR/bin/moc ]; then
  echo "*** ERROR: unable to locate Qt MOC in $QTDIR/bin"
  exit 1
fi

if [ ! -r $QTDIR/bin/uic ]; then
  echo "*** ERROR: unable to locate Qt UIC in $QTDIR/bin"
  exit 1
fi

BUILD=build.$PLATFORM
BIN=bin.$PLATFORM

if ( gmake -v >/dev/null 2>/dev/null ); then
  MAKE_PRG=gmake
else
  MAKE_PRG=make
fi

mkdir -p $BUILD
cp -f src/Makefile $BUILD/Makefile
cp -f config/Makefile.conf.$PLATFORM $BUILD/Makefile.conf

echo "" >$BUILD/Makefile.env
echo "SOURCE=$SOURCE" >>$BUILD/Makefile.env
echo "QTDIR=$QTDIR" >>$BUILD/Makefile.env

echo "Running build."
echo "To rebuild: cd $BUILD ; $MAKE_PRG all"

cd $BUILD
rm -f $EXEC_NAME
$MAKE_PRG all
cd ..

if [ ! -r $BUILD/$EXEC_NAME ]; then
  echo "*** ERROR: build failed."
  exit 1
fi

echo "Installing binary: cp $BUILD/$EXEC_NAME $BIN"
mkdir -p $BIN
cp $BUILD/$EXEC_NAME $BIN

echo "Build done."
echo "Final binary is ready in ./$BIN/$EXEC_NAME."
echo "To run it, you may need to add the following to your LD_LIBRARY_PATH:"
echo "$QTDIR/lib"

LD_LIBRARY_PATH=$QTDIR/lib:$LD_LIBRARY_PATH
./$BIN/$EXEC_NAME -v
./$BIN/$EXEC_NAME -h


