
/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layLayoutViewConfigPages.h"

#include "LayoutViewConfigPage.h"
#include "LayoutViewConfigPage2.h"
#include "LayoutViewConfigPage3.h"

#include "layWidgets.h"
#include "dbHershey.h"

namespace lay
{

// ------------------------------------------------------------
//  Declaration of the configuration options
//  The configuration pages are declared via two "dummy" plugins

const std::string cfg_background_color ("background-color");

const std::string cfg_min_inst_label_size ("min-inst-label-size");
const std::string cfg_cell_box_text_font ("inst-label-font");
const std::string cfg_cell_box_text_transform ("inst-label-transform");
const std::string cfg_cell_box_color ("inst-color");
const std::string cfg_cell_box_visible ("inst-visible");
const std::string cfg_text_color ("text-color");
const std::string cfg_text_visible ("text-visible");
const std::string cfg_text_lazy_rendering ("text-lazy-rendering");
const std::string cfg_apply_text_trans ("apply-text-trans");
const std::string cfg_default_text_size ("default-text-size");
const std::string cfg_text_font ("text-font");
const std::string cfg_fit_new_cell ("fit-new-cell");

// ------------------------------------------------------------
//  LayoutConfigPage implementation

LayoutViewConfigPage::LayoutViewConfigPage (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::LayoutViewConfigPage ();
  mp_ui->setupUi (this);

  mp_background_color_cbtn = new lay::ColorButton (mp_ui->bkgnd_color_pb);
}

LayoutViewConfigPage::~LayoutViewConfigPage ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
LayoutViewConfigPage::setup (lay::Plugin *root)
{
  std::string value;

  QColor color;
  root->config_get (cfg_background_color, color, ColorConverter ());
  mp_background_color_cbtn->set_color (color);
}

void 
LayoutViewConfigPage::commit (lay::Plugin *root)
{
  root->config_set (cfg_background_color, mp_background_color_cbtn->get_color (), lay::ColorConverter ());
}

// ------------------------------------------------------------
//  LayoutConfigPage2 implementation

LayoutViewConfigPage2::LayoutViewConfigPage2 (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::LayoutViewConfigPage2 ();
  mp_ui->setupUi (this);

  mp_text_color_cbtn = new lay::ColorButton (mp_ui->text_color_pb);
  mp_box_color_cbtn = new lay::ColorButton (mp_ui->cell_box_color_pb);
}

LayoutViewConfigPage2::~LayoutViewConfigPage2 ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
LayoutViewConfigPage2::setup (lay::Plugin *root)
{
  QColor color;
  root->config_get (cfg_text_color, color, ColorConverter ());
  mp_text_color_cbtn->set_color (color);

  root->config_get (cfg_cell_box_color, color, ColorConverter ());
  mp_box_color_cbtn->set_color (color);

  bool flag = false;
  root->config_get (cfg_cell_box_text_transform, flag);
  mp_ui->cell_xform_text_cbx->setChecked (flag);

  root->config_get (cfg_apply_text_trans, flag);
  mp_ui->text_apply_trans_cbx->setChecked (flag);

  root->config_get (cfg_text_visible, flag);
  mp_ui->text_group->setChecked (flag);

  root->config_get (cfg_text_lazy_rendering, flag);
  mp_ui->text_lazy_rendering_cbx->setChecked (flag);

  root->config_get (cfg_cell_box_visible, flag);
  mp_ui->cell_group->setChecked (flag);

  int font = 0;
  root->config_get (cfg_cell_box_text_font, font);
  if (mp_ui->cell_font_cb->count () == 0) {
    std::vector<std::string> ff = db::Hershey::font_names ();
    for (std::vector<std::string>::const_iterator f = ff.begin (); f != ff.end (); ++f) {
      mp_ui->cell_font_cb->insertItem (f->c_str ());
    }
  }
  mp_ui->cell_font_cb->setCurrentItem (font);

  root->config_get (cfg_text_font, font);
  if (mp_ui->text_font_cb->count () == 0) {
    std::vector<std::string> ff = db::Hershey::font_names ();
    for (std::vector<std::string>::const_iterator f = ff.begin (); f != ff.end (); ++f) {
      mp_ui->text_font_cb->insertItem (f->c_str ());
    }
  }
  mp_ui->text_font_cb->setCurrentItem (font);

  double s = 0.0;
  root->config_get (cfg_default_text_size, s);
  mp_ui->text_def_size_edit->setText (tl::to_string (s).c_str ());

  int n = 0;
  root->config_get (cfg_min_inst_label_size, n);
  mp_ui->cell_min_size_for_label_edit->setText (tl::to_string (n).c_str ());
}

void 
LayoutViewConfigPage2::commit (lay::Plugin *root)
{
  root->config_set (cfg_text_color, mp_text_color_cbtn->get_color (), ColorConverter ());
  root->config_set (cfg_cell_box_color, mp_box_color_cbtn->get_color (), ColorConverter ());
  root->config_set (cfg_cell_box_visible, mp_ui->cell_group->isChecked ());

  root->config_set (cfg_cell_box_text_transform, mp_ui->cell_xform_text_cbx->isChecked ());
  root->config_set (cfg_apply_text_trans, mp_ui->text_apply_trans_cbx->isChecked ());
  root->config_set (cfg_text_visible, mp_ui->text_group->isChecked ());
  root->config_set (cfg_text_lazy_rendering, mp_ui->text_lazy_rendering_cbx->isChecked ());

  root->config_set (cfg_cell_box_text_font, mp_ui->cell_font_cb->currentItem ());
  root->config_set (cfg_text_font, mp_ui->text_font_cb->currentItem ());

  try {
    double s;
    tl::from_string (mp_ui->text_def_size_edit->text ().ascii (), s);
    root->config_set (cfg_default_text_size, s);
  } catch (...) { }

  try {
    int n;
    tl::from_string (mp_ui->cell_min_size_for_label_edit->text ().ascii (), n);
    root->config_set (cfg_min_inst_label_size, n);
  } catch (...) { }
}

// ------------------------------------------------------------
//  LayoutConfigPage3 implementation

LayoutViewConfigPage3::LayoutViewConfigPage3 (QWidget *parent)
  : lay::ConfigPage (parent)
{
  mp_ui = new Ui::LayoutViewConfigPage3 ();
  mp_ui->setupUi (this);
}

LayoutViewConfigPage3::~LayoutViewConfigPage3 ()
{
  delete mp_ui;
  mp_ui = 0;
}

void 
LayoutViewConfigPage3::setup (lay::Plugin *root)
{
  bool flag = true;
  root->config_get (cfg_fit_new_cell, flag);
  mp_ui->fit_new_cell_cbx->setChecked (flag);
}

void 
LayoutViewConfigPage3::commit (lay::Plugin *root)
{
  root->config_set (cfg_fit_new_cell, mp_ui->fit_new_cell_cbx->isChecked ());
}

// ------------------------------------------------------------
//  The dummy plugin declarations to register the configuration options

class LayoutViewPluginDeclaration
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_background_color, "auto"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Background");
    return new LayoutViewConfigPage (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl (new LayoutViewPluginDeclaration (), 2001, "LayoutViewPlugin(1)");



class LayoutViewPluginDeclaration2
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_min_inst_label_size, "16"));
    options.push_back (std::pair<std::string, std::string> (cfg_cell_box_text_font, "0"));
    options.push_back (std::pair<std::string, std::string> (cfg_cell_box_text_transform, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_cell_box_color, "auto"));
    options.push_back (std::pair<std::string, std::string> (cfg_cell_box_visible, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_text_color, "auto"));
    options.push_back (std::pair<std::string, std::string> (cfg_text_visible, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_text_lazy_rendering, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_apply_text_trans, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_default_text_size, "0.1"));
    options.push_back (std::pair<std::string, std::string> (cfg_text_font, "0"));
    options.push_back (std::pair<std::string, std::string> (cfg_fit_new_cell, "true"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Display");
    return new LayoutViewConfigPage2 (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl2 (new LayoutViewPluginDeclaration2 (), 2000, "LayoutViewPlugin(2)");



class LayoutViewPluginDeclaration3
  : public lay::PluginDeclaration
{
public:
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_fit_new_cell, "true"));
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const 
  {
    title = tl::translate ("Misc.");
    return new LayoutViewConfigPage3 (parent); 
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl3 (new LayoutViewPluginDeclaration3 (), 2002, "LayoutViewPlugin(3)");

}

